
# CONFIG is inserted above this line during INSTALL

trap : 2 3

PATH=/bin:$PATH
export PATH

# first argument is operation to be performed:
# when saved, we shift it off

OPERATION=$1
shift


# first we handle 'cancel'
# $1 is ident, $2 is group name.

if [ "$OPERATION" = "cancel" ] ; then
	$INEWS -t 'cmsg cancel '"$1" -n "$2" < /dev/null > /tmp/nn$$c 2>&1
	x=$?
	if [ $x != 0 ]; then
		echo ''
		cat /tmp/nn$$c
		sleep 2
	fi
	rm -f /tmp/nn$$c
	exit $x
fi

WORK="$1"
ED_LINE="$2"
FIRST_ACTION="$3"
RECORD="$4"

TRACE=${WORK}T
COPY=""

if [ "${FIRST_ACTION}" = "edit" ] ; then
	COPY=${WORK}C
	cp $WORK $COPY
fi

# loop until sent or aborted.

loop=true
prompt=false
while $loop ; do
	if $prompt ; then
		echo ''
		awk 'END{printf "Action: (a)bort (e)dit (o)k (s)end: "}' < /dev/null
		read act
	else
		act="${FIRST_ACTION}"
		prompt=true
	fi

	case $act in
	a*) 
		rm -f $WORK $COPY
		exit 22
		;;
	e*)
		# call editor to enter at line $2 of work file

		case `basename "${EDITOR-vi}"` in
		vi|emacs )
			# Berkeley vi display editor
			# GNU emacs disply editor
			${EDITOR-vi} +${ED_LINE} $WORK
			;;
		ded )
			# QMC ded display editor
			$EDITOR -l${ED_LINE} $WORK
			;;
		uemacs )
			# micro emacs
			$EDITOR -g${ED_LINE} $WORK
			;;
		* )
			# Unknown editor
			$EDITOR $WORK
			;;
		esac
		;;

	o*|s*)
		loop=false
		;;

	esac
done

if [ -n "$COPY" ] ; then
	if [ -s $WORK ] ; then
		if cmp -s $WORK $COPY ; then
			rm -f $WORK $COPY
			exit 22
		fi
	else
		rm -f $WORK $COPY
		exit 22
	fi
		
	rm -f $COPY
fi

case "$OPERATION" in
reply|forward|mail)
	if $APPENDSIG ; then
	    if [ -f $HOME/.signature ] ; then
		awk 'END{printf "Append .signature? (y) : "}' < /dev/null
		read ans
		case $ans in
		''|y*|Y*)
			echo "--" >> $WORK
			cat $HOME/.signature >> $WORK
			;;
		esac
	    fi
	fi
	;;
follow|post)
	echo "Be patient! Your new article will not show up immediately."
	sleep 2
	;;
esac

{
	trap 'echo SIGNAL' 1 2 3

	case "$OPERATION" in

	reply|forward|mail)
		grep -v "^Orig-To: " $WORK | $RECMAIL
		x=$?
		;;

	follow|post)
		grep -v "^Orig-To: " $WORK | $INEWS -h
		x=$?
		# wait for inews to finish
		sleep 60
		;;

	*)
		echo "Invalid operation: $OPERATION -- help"
		OPERATION="nn response operation"
		x=1
		cat > /dev/null
		;;

	esac > $TRACE 2>&1


	if [ 0"$x" -ne 0 -o -s $TRACE ] ; then
		if [ -s $HOME/dead.letter ] ; then
			cat $HOME/dead.letter >> $HOME/dead.letters
			echo '' >> $HOME/dead.letters
		fi
		grep -v "^Orig-To: " $WORK > $HOME/dead.letter
		{
		echo "To: ${LOGNAME-$USER}"
		echo "Subject: $OPERATION failed"
		echo ''
		cat $TRACE
		echo ''
		echo 'Your response has been saved in ~/dead.letter'
		echo ''
		echo 'Your article/letter follows:'
		cat $WORK
		} | $RECMAIL
		
	elif [ -n "${RECORD}" ] ; then
		{
		# keep a copy of message in $RECORD (in mail format)
		set `date`
		if [ $3 -gt 9 ] ; then
			echo From ${LOGNAME-$USER} $1 $2 $3 $4 $6 $7
		else
			echo From ${LOGNAME-$USER} $1 $2 ' '$3 $4 $6 $7
		fi
		echo "From: ${LOGNAME-$USER}"
		grep -v '^Orig-To: ' $WORK
		echo '' 
		} >> "$RECORD"
	fi

	rm -f $WORK $TRACE

} > /dev/null 2>&1 &

exit 0
