/*
 *	save selected articles (if any) between invocations
 */

#include "config.h"
#include "articles.h"

#define	SLMAGIC	0x536c6374
#define GNAME_LGT 32

static struct sel_header {
    char		sl_group[GNAME_LGT];
    int32		sl_magic;
    int32		sl_count;
    int32		sl_first;
    int32		sl_last;
} header;

#ifndef NETWORK_DATABASE
#undef ntohl
#undef htonl
#undef NETWORK_BYTE_ORDER
#define ntohl(x) (x)
#define htonl(x) (x)
#define NETWORK_BYTE_ORDER
#endif

struct sel_art {
    int32		sl_number;
    int32		sl_fpos;
    int32		sl_flag;
};


save_selection(gh, first, last)
group_header *gh;
article_number first, last;
{
    register article_header *ah, **ahp;
    register int art;
    register int save_count, seen_count, flags;
    struct sel_art elem;
    FILE *f;
    char buffer[16];

    if (gh->group_flag & (G_FOLDER | G_READ)) return;

    for (save_count = seen_count = 0, art = n_articles; --art >= 0; ) {
	flags = articles[art]->flag; 
	if (flags & A_SEEN) seen_count++;
	if (flags & A_SELECT) save_count++;
    }
    
    if (save_count == 0 && 
	(seen_count == 0 || seen_count == n_articles)) return;

    sprintf(buffer, "S.%d", (int)(gh->group_num));
    f = open_file(relative(nn_directory, buffer), OPEN_CREATE|MUST_EXIST);
    
    strncpy(header.sl_group, gh->group_name, GNAME_LGT);
    header.sl_magic = htonl(SLMAGIC);
    header.sl_count = htonl(save_count);
    header.sl_first = htonl(first);
    header.sl_last  = htonl(last);
    
    fwrite(&header, sizeof(header), 1, f);

    unsort_articles(1);
    
    for (ahp = articles, art = 0; art < n_articles; ahp++, art++) {
	ah = *ahp;
	if (elem.sl_flag = htonl(ah->flag & (A_SELECT | A_SEEN))) {
	    
	    elem.sl_number = htonl(ah->a_number);
	    elem.sl_fpos   = htonl(ah->fpos);
	    
	    fwrite(&elem, sizeof(elem), 1, f);
	}
    }
    
    fclose(f);

    gh->group_flag |= G_SELECTION;
}


static FILE *sel_file;

has_selection(gh, first_p, last_p)
group_header *gh;
article_number *first_p, *last_p;
{
    FILE *f;
    char buffer[16];

    if (gh->group_flag & (G_FOLDER | G_READ)) return 0;
    
    sprintf(buffer, "S.%d", (int)(gh->group_num));    
    f = open_file(relative(nn_directory, buffer), OPEN_READ|OPEN_UNLINK);
    if (f == NULL) return 0;

    if (fread(&header, sizeof(header), 1, f) != 1
	|| ntohl(header.sl_magic) != SLMAGIC
	|| strncmp(header.sl_group, gh->group_name, GNAME_LGT)) {
	fclose(f);
	return 0;
    }

    if ((gh->group_flag & G_SELECTION) == 0) {
	clrdisp();
	raw();
	prompt("Group %s: use old selections (%s)? ", 
	       gh->group_name, date_time(m_time(f)));
	if (yes(0) <= 0) {
	    fclose(f);
	    return 0;
	} 
    }
    
#ifndef NETWORK_BYTE_ORDER
    header.sl_first = ntohl(header.sl_first);
    header.sl_last = ntohl(header.sl_last);
#endif
    *first_p = header.sl_first;
    *last_p = header.sl_last;
    sel_file = f;
    
    return 1;
}


do_selections(ok)
int ok;
{
    register long art;
    struct sel_art elem;
    register article_header *ah, **ahp;

    if (!ok) goto out;
    
    elem.sl_number = -1;
    
    for (ahp = articles, art = 0; art < n_articles; ahp++, art++) {
	ah = *ahp;

	if (ah->a_number > header.sl_last) break;
    
	while (ah->a_number > elem.sl_number) {
	    if (fread(&elem, sizeof(elem), 1, sel_file) != 1) goto out;
#ifndef NETWORK_BYTE_ORDER
	    elem.sl_number = ntohl(elem.sl_number);
#endif
	}
	
#ifndef NETWORK_BYTE_ORDER
	elem.sl_fpos = ntohl(elem.sl_fpos);
#endif
	while (ah->a_number == elem.sl_number && ah->fpos > elem.sl_fpos) {
	    if (fread(&elem, sizeof(elem), 1, sel_file) != 1) goto out;
#ifndef NETWORK_BYTE_ORDER
	    elem.sl_number = ntohl(elem.sl_number);
	    elem.sl_fpos = ntohl(elem.sl_fpos);
#endif
	}
	
	if (ah->a_number != elem.sl_number || ah->fpos != elem.sl_fpos) 
	    continue;
	
	ah->flag |= ntohl(elem.sl_flag);
    }
    
out:
	
    fclose(sel_file);
}

