/*
 *	Variabel setting and display
 */

#include "config.h"

import in_init;

import char			/* string variables */
    *default_save_file,
    *header_lines,
    *folder_directory,
    included_mark[],
    *mail_box,
    *mail_record,
    *news_record,
#ifdef NNTP
    nntp_server[],
#endif
    *pager,
    *patch_cmd,
    printer[],
    *save_counter_format;

import int			/* boolean variables */
    conf_append,
    conf_dont_sleep,
    delay_redraw,
    do_kill_handling,
    dont_sort_articles,
    dont_sort_folders,
    dont_split_digests,
    fmt_rptsubj,
    keep_rc_backup,
    long_menu,
    macro_debug,
    mark_overlap,
    monitor_mode,
    novice,
    quick_save,
    save_report,
    show_article_date,
    show_current_time,
    silent,
    use_mail_folders,
    use_newsrc;

import int			/* integer variables */
    also_read_articles,
    article_limit,
    Debug,
    first_page_lines,
    fmt_linenum,
    Lines,
    overlap,
    preview_window,
    retry_on_error;

import char			/* key strokes */
    comp1_key,
    comp2_key,
    help_key,
    erase_key,
    delword_key,
    kill_key;

#define	V_STRING	0
#define V_BOOLEAN	1
#define	V_INTEGER	2
#define V_KEY		3
#define	V_SPECIAL	4

struct variable_defs {
    char *var_name;
    char var_type;
    char var_op;
    char **var_addr;
} variables[] = {
    "backup",		V_BOOLEAN,	3,	(char **)&keep_rc_backup,
    "comp1_key",	V_KEY,		0,	(char **)&comp1_key,
    "comp2_key",	V_KEY,		0,	(char **)&comp2_key,
    "confirm",		V_BOOLEAN,	0,	(char **)&conf_dont_sleep,
    "confirm-append",	V_BOOLEAN,	0,	(char **)&conf_append,
    "date",		V_BOOLEAN,	0,	(char **)&show_article_date,
    "debug",		V_INTEGER,	0,	(char **)&Debug,
    "default-save-file",V_STRING,	0,	(char **)&default_save_file,
    "delay-redraw",	V_BOOLEAN,	0,	(char **)&delay_redraw,
    "erase-key",	V_KEY,		0,	(char **)&erase_key,
    "expert",		V_BOOLEAN,	4,	(char **)&novice,
    "folder",		V_STRING,	2,	(char **)&folder_directory,
    "fsort",		V_BOOLEAN,	2,	(char **)&dont_sort_folders,
    "header-lines",	V_STRING,	0,	(char **)&header_lines,
    "help-key",		V_KEY,		0,	(char **)&help_key,
    "included-mark",	V_STRING,	1,	(char **)included_mark,
    "kill",		V_BOOLEAN,	0,	(char **)&do_kill_handling,
    "kill-key",		V_KEY,		0,	(char **)&kill_key,
    "layout",		V_INTEGER,	1,	(char **)&fmt_linenum,
    "limit",		V_INTEGER,	0,	(char **)&article_limit,
    "lines",		V_INTEGER,	1,	(char **)&Lines,
    "long-menu",	V_BOOLEAN,	1,	(char **)&long_menu,
    "macro-debug",	V_BOOLEAN,	0,	(char **)&macro_debug,
    "mail",		V_STRING,	0,	(char **)&mail_box,
    "mail-format",	V_BOOLEAN,	0,	(char **)&use_mail_folders,
    "mail-record",	V_STRING,	2,	(char **)&mail_record,
    "mark-overlap",	V_BOOLEAN,	0,	(char **)&mark_overlap,
    "monitor",		V_BOOLEAN,	0,	(char **)&monitor_mode,
    "news-record",	V_STRING,	2,	(char **)&news_record,
    "newsrc",		V_BOOLEAN,	0,	(char **)&use_newsrc,
#ifdef NNTP
    "nntp-server",	V_STRING,	1,	(char **)nntp_server,
#endif
    "old",		V_SPECIAL,	2,	(char **)NULL,
    "overlap",		V_INTEGER,	0,	(char **)&overlap,
    "pager",		V_STRING,	0,	(char **)&pager,
    "patch-cmd",	V_STRING,	0,	(char **)&patch_cmd,
    "printer",		V_STRING,	1,	(char **)printer,
    "quick-save",	V_BOOLEAN,	0,	(char **)&quick_save,
    "record",		V_SPECIAL,	1,	(char **)NULL,
    "repeat",		V_BOOLEAN,	0,	(char **)&fmt_rptsubj,
    "retry-on-error",	V_INTEGER,	0,	(char **)&retry_on_error,
    "save-counter",	V_STRING,	0,	(char **)&save_counter_format,
    "save-report",	V_BOOLEAN,	0,	(char **)&save_report,
    "silent",		V_BOOLEAN,	0,	(char **)&silent,
    "sort",		V_BOOLEAN,	2,	(char **)&dont_sort_articles,
    "split",		V_BOOLEAN,	4,	(char **)&dont_split_digests,
    "stop",		V_INTEGER,	0,	(char **)&first_page_lines,
    "time",		V_BOOLEAN,	0,	(char **)&show_current_time,
    "window",		V_INTEGER,	1,	(char **)&preview_window,
    "word-key",		V_KEY,		0,	(char **)&delword_key
};

#define TABLE_SIZE	(sizeof(variables)/sizeof(struct variable_defs))

#define INT_VAR		(*((int *)(var->var_addr)))
#define BOOL_VAR	(*((int *)(var->var_addr)))
#define STR_VAR		(*(var->var_addr))
#define CBUF_VAR	((char *)(var->var_addr))
#define KEY_VAR		(*((char *)(var->var_addr)))


static struct variable_defs *lookup_variable(variable)
char *variable;
{
    register struct variable_defs *var;
    register i, j, k, t;
    
    i = 0; j = TABLE_SIZE - 1; 
    
    while (i <= j) {
	k = (i + j) / 2;
	var = &variables[k];
	
	if ( (t=strcmp(variable, var->var_name)) > 0) 
	    i = k+1;
	else
	if (t < 0)
	    j = k-1;
	else
	    return var;
    }

    init_message("unknown variable: %s", variable);
    return NULL;
}    


set_variable(variable, on, val_string)
char *variable;
int on;
char *val_string;
{
    long value;
    register struct variable_defs *var;

    if (strncmp(variable, "no", 2) == 0) {
	on = !on;
	variable += 2;
    }
    
    if ((var = lookup_variable(variable)) == NULL) 
	return 0;
    
    if (!on || val_string == NULL)
	value = 0;
    else	
	value = atol(val_string);

    switch (var->var_type) {
	
     case V_STRING:

	switch (var->var_op) {
	 case 0:
	    if (val_string) STR_VAR = copy_str(val_string);
	    break;

	 case 1:
	    if (val_string) strcpy(CBUF_VAR, val_string);
	    break;

	 case 2:
	    adjust(val_string);
	    if (val_string) STR_VAR = home_relative(val_string);
	    break;
	}
	break;
	
     case V_BOOLEAN:
	
	switch (var->var_op) {
	 case 0:
	    BOOL_VAR = on;
	    break;
	    
	 case 1:
	    BOOL_VAR = on;
	    return 1;
	    
	 case 2:
	    if (BOOL_VAR == on) {
		BOOL_VAR = !on;
		if (!in_init) {
		    if (BOOL_VAR)
			unsort_articles(1);
		    else
			sort_articles();
		    return 1;
		}
	    }	
	    break;
	    
	 case 3:
	    if (!in_init) {
		msg("'%s' can only be set in the init file", variable);
		break;
	    }
	    BOOL_VAR = on;
	    break;	    

	 case 4:
	    BOOL_VAR = !on;
	    break;
	}
	break;

     case V_INTEGER:

	switch (var->var_op) {
	 case 0:
	    INT_VAR = value;	    
	    break;
	    
	 case 1:
	    INT_VAR = value; 
	    return 1;
	}
	break;
	
     case V_KEY:
	switch (var->var_op) {
	 case 0:
	    if (val_string) {
		if (*val_string) adjust(val_string + 1); /* #N is valid */
		KEY_VAR = parse_key(val_string);
	    }
	    break;
	}
	break;
	
     case V_SPECIAL:
	
	switch (var->var_op) {
	 case 1:
	    if (val_string) {
		adjust(val_string);
		news_record = home_relative(val_string);
		mail_record = news_record;
	    }
	    break;
	    
	 case 2:
	    also_read_articles = on;
	    article_limit = value;
	    break;
	}	    
	break;
    }
    return 0;
}
    
static adjust(str)
register char *str;
{
    if (str == NULL) return;
    while (*str && !isspace(*str) && *str != '#') str++;
    *str = NUL;
}


disp_variables()
{
    char *str, *key_name();
    int i, b;
    register struct variable_defs *var;
    
    if (in_init) return;

    pg_init(0, 1);
    
    clrdisp();
    pg_next();
    so_printf("\1Variable settings:\1");

    for (var = variables; var < &variables[TABLE_SIZE]; var++) {
	switch (var->var_type) {
	 case V_STRING:
	    str = (var->var_op == 1) ? CBUF_VAR : STR_VAR;
	    if (str == NULL || *str == NUL) break;
	    
	    if (pg_next() < 0) goto out;
	    printf("%-20.20s = \"%s\"\n", var->var_name, str);
	    break;

	 case V_BOOLEAN:
	    if (pg_next() < 0) goto out;
	    b = BOOL_VAR;
	    if (var->var_op == 2 || var->var_op == 4) b = !b;
	    printf("%s%s\n", b ? "" : "no", var->var_name);
	    break;
	    
	 case V_INTEGER:
	    i = INT_VAR;
	    if (i == 0) break;
	    if (pg_next() < 0) goto out;
	    printf("%-20.20s = %d\n", var->var_name, i);
	    break;
	    
	 case V_KEY:
	    if (pg_next() < 0) goto out;
	    printf("%-20.20s = %s\n", var->var_name, key_name(KEY_VAR));
	    break;
	    
	 case V_SPECIAL:
	    switch (var->var_op) {
	     case 1:
		break;
	     case 2:
		if (also_read_articles) {
		    if (pg_next() < 0) goto out;
		    printf("%-20.20s = %d\n", var->var_name, article_limit);
		}
		break;
	    }
	    break;
	}
    }
    
out:
    pg_end();
}

toggle_variable(variable)
char *variable;
{
    register struct variable_defs *var;
    
    if ((var = lookup_variable(variable)) == NULL) return;
    if (var->var_type != V_BOOLEAN) {
	init_message("variable %s is not boolean", variable);
	return;
    }
    
    BOOL_VAR = !BOOL_VAR;
}


test_variable(expr)
char *expr;
{
    char *variable;
    register struct variable_defs *var;
    int res = -1;
    
    variable = expr;
    if ((expr = strchr(variable, '=')) == NULL) 
	goto err;
    
    *expr++ = NUL;
    
    if ((var = lookup_variable(variable)) == NULL) {
	msg("testing unknown variable %s=%s", variable, expr);
	goto out;
    }    

    switch (var->var_type) {
	
     case V_BOOLEAN:
	res = BOOL_VAR;
	
	if (strcmp(expr, "on") == 0) break;
	if (strcmp(expr, "off") == 0) {
	    res = !res;
	    break;
	}
	msg("boolean variables must be tested =on or =off");
	break;
	
     case V_INTEGER:
	res = (INT_VAR == atoi(expr)) ? 1 : 0;
	break;
	
     default:
	msg("%s: cannot only test boolean and integer variables", variable);
	break;
    }
 out:
    *--expr = '=';
 err:
    return res;
}


var_completion(path, index)
char *path;
int index;
{
    static char *head, *tail = NULL;
    static int len;
    static struct variable_defs *var, *help_var;

    if (index < 0) return 0;

    if (path) {
	head = path;
	tail = path + index;
	while (*head && isspace(*head)) head++;
	help_var = var = variables;
	len = tail - head;
	
	return 1;
    }
    
    if (index) {
	list_completion((char *)NULL);
	
	for (;; help_var++) {
	    if (help_var >= &variables[TABLE_SIZE]) {
		help_var = variables;
		break;
	    }
	    
	    index = strncmp(help_var->var_name, head, len);
	    if (index < 0) continue;
	    if (index > 0) {
		help_var = variables;
		break;
	    }
	    if (list_completion(help_var->var_name) == 0) break;
	}
	fl;
	return 1;
    }

    for (; var < &variables[TABLE_SIZE]; var++) {
	if (len == 0) 
	    index = 0;
	else
	    index = strncmp(var->var_name, head, len);
	if (index < 0) continue;
	if (index > 0) break;
	sprintf(tail, "%s ", var->var_name + len);
	var++;
	return 1;
    }
    return 0;
}

